<?php
namespace Ep\Gath\DataStore;

use Ep\Gath\Plugin\Products;
use WC_Product_Variation_Data_Store_CPT;

/**
 * Class VariationDataStore
 *
 * @package Ep\Gath\DataStore
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class VariationDataStore extends WC_Product_Variation_Data_Store_CPT
{
    /**
     * Reads a product from the database and sets its data to the class.
     *
     * @since 3.0.0
     * @param \WC_Product $product
     * @throws Exception
     */
    public function read( &$product ) {
        $product->set_defaults();

        if ( ! $product->get_id() || ! ( $post_object = get_post( $product->get_id() ) ) || ! in_array( $post_object->post_type, array( 'product', 'product_variation' ) ) ) {
            return;
        }

        /** @var Products $gath */
        $gath = apply_filters('ep_gath_products', null);

        $product->set_props( array(
            'name'            => $post_object->post_title,
            'slug'            => $post_object->post_name,
            'date_created'    => 0 < $post_object->post_date_gmt ? wc_string_to_timestamp( $post_object->post_date_gmt ) : null,
            'date_modified'   => 0 < $post_object->post_modified_gmt ? wc_string_to_timestamp( $post_object->post_modified_gmt ) : null,
            'status'          => $post_object->post_status,
            'menu_order'      => $post_object->menu_order,
            'reviews_allowed' => 'open' === $post_object->comment_status,
            'parent_id'       => $post_object->post_parent,
        ) );

        // The post parent is not a valid variable product so we should prevent this.
        if ( $product->get_parent_id( 'edit' ) && 'product' !== get_post_type( $product->get_parent_id( 'edit' ) ) ) {
            $product->set_parent_id( 0 );
        }

        $this->read_downloads( $product );
        $this->read_product_data( $product );
        $this->read_extra_data( $product );

        $result = $gath->getRawResult($product->get_id());
        $attributes = [];

        foreach ($result['Attributes'] ?? [] as $attribute) {
            $name = 'attribute_' . wc_variation_attribute_name($attribute['name']);
            $attributes[$name] = $attribute['value'];
        }
        
        $product->set_attributes( $attributes);

        /**
         * If a variation title is not in sync with the parent e.g. saved prior to 3.0, or if the parent title has changed, detect here and update.
         */
        $new_title = $this->generate_product_title( $product );

        if ( $post_object->post_title !== $new_title ) {
            $product->set_name( $new_title );
            clean_post_cache( $product->get_id() );
        }

        // Set object_read true once all data is read.
        $product->set_object_read( true );
    }
}